package org.jtag.mp3;

import java.io.*;
import java.util.*;

import org.jtag.id3.*;


public class mp3Info {
    // Vector to stare Mp3 header info
    private Vector mp3Vector = new Vector();
    private Hashtable mp3Hash = new Hashtable();

    // public section of member variables
    public long fileLength;
    public String fileName;
    public String directoryName;
    public boolean hasIdTag;

    // ID3 Tags
    public String songTitle;
    public String artist;
    public String album;
    public String year;
    public String comment;
    public String genre;

    // MP3 Tags
    public String mpegVersion = new String();
    public String mpegLayer = new String();
    public String protectionBit = new String();
    public String bitrate = new String();
    public String frequency = new String();
    public String paddingBit = new String();
    public String privateBit = new String();
    public String mpegMode = new String();
    public String mpegModeExtension = new String();
    public String copyrightBit = new String();
    public String originalHome = new String();
    public String emphasis = new String();
    public String playLength = new String();
    public String playLengthMin = new String();
    public String playLengthSec = new String();

    // private section of member variables
    private String fullFileName;
    private int tagOffset = 128;
    private String tagString;
    private byte tagBytes[] = new byte[3];
    private byte songTitleBytes[] = new byte[30];
    private byte artistBytes[] = new byte[30];
    private byte albumBytes[] = new byte[30];
    private byte yearBytes[] = new byte[4];
    private byte commentBytes[] = new byte[30];
    private byte genreBytes[] = new byte[1];

    private String Switch[] = {"OFF", "ON"};
    private String mpegVersionString[] = {"MPEG 2", "MPEG 1"};
    private String mpegLayerString[] = {"Not def.", "Layer 3", "Layer 2",
            "Layer 1"};
    private String mpegModeString[] = {"Stereo", "Joint-Stereo", "Dual-Channel",
            "Mono"};
    private String mpegEmphasisString[] = {"none", "50/15ms", "-?-", "CITTj.17"};
    // mpegBitrate[MPEG-Version][Layer][Bitrate]
    // MPEG-Version:   0 = MPEG 2
    //					        1 = MPEG 1
    // Laver:			    0 = Layer 3
    //					        1 = Layer 2
    //					        2 = Layer 1
    private int mpegBitrateString[][][] = { { {0, 8, 16, 24, 32, 64, 80,
                    56, 64, 128, 160, 112, 128, 256, 320, 0}, {0, 32, 48, 56,
                    64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0}, {0,
                    32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384,
                    416, 448, 0}
            }, { {0, 32, 40, 48, 56, 64, 80, 96, 112, 128,
                    160, 192, 224, 256, 320, 0}, {0, 32, 48, 56, 64, 80, 96,
                    112, 128, 160, 192, 224, 256, 320, 384, 0}, {0, 32, 64, 96,
                    128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0}
            }
        };
    // frequencyString[MPEG-Version][frequency-bits]
    // MPEG-Version:   0 = MPEG 2
    //					1 = MPEG 1
    private int mpegFrequencyString[][] = { {22050, 24000, 16000, 0}, {44100, 48000, 32000, 0}
        };

    private int mpegModeExtensionString[][] = { {0, 4, 8, 16}, {4, 8, 12,
                16}
        };

    public mp3Info(String dir, String file) {
        // init member variables
        fileLength = 0;
        fileName = new String(file);
        directoryName = new String(dir);
        fullFileName = new String(directoryName + File.separator + fileName);

        // Assume id3Tag is there
        hasIdTag = true;

        // get id3 information
        getId3Info();

        // get mp3Info
        getMp3Info();

        // Make mp3 hash
        makeMp3Hash();
    }

    private void getId3Info() {
        try {
            // Open the file
            File fileObj = new File(fullFileName);
            FileInputStream inputStream = new FileInputStream(fileObj);

            fileLength = fileObj.length();

            // If the file is smaller than tagOffset, there is no
            // ID 3 tag
            if (fileLength - tagOffset <= 0) {
                hasIdTag = false;
                return;
            }
            // Go to TAG location
            long skipped = inputStream.skip(fileLength - tagOffset);

            // Check if tag "TAG" is available
            int read = inputStream.read(tagBytes);

            tagString = new String(tagBytes);

            if (tagString.compareTo("TAG") != 0) {
                hasIdTag = false;
            } else {
                // Get the ID3 info
                inputStream.read(songTitleBytes);
                songTitle = new String(songTitleBytes);

                inputStream.read(artistBytes);
                artist = new String(artistBytes);

                inputStream.read(albumBytes);
                album = new String(albumBytes);

                inputStream.read(yearBytes);
                year = new String(yearBytes);

                inputStream.read(commentBytes);
                comment = new String(commentBytes);

                inputStream.read(genreBytes);

                //System.out.println("GenreBytes: " + genreBytes[0]);
                // If genre > 114, genre is empty. Otherwise set genre
                id3Genre cGenre = new id3Genre();
                String[] sGenre = cGenre.getList();

                if (genreBytes[0] < sGenre.length && genreBytes[0] >= 0)
                    genre = sGenre[genreBytes[0]];
                else
                    genre = new String("");

            }

            // Close the input stream
            inputStream.close();

        } catch (Exception e) {
            System.err.println("Exception: " + e + "in file " + fileName);
            System.err.println("DUMP:");
            System.err.println("albumBytesLength: " + albumBytes.length);
            System.err.println("\tFile-Length: " + fileLength);
            System.err.println("\tSong-Title : " + songTitle);
            System.err.println("\tArtist     : " + artist);
            System.err.println("\tAlbum      : " + album);
            System.err.println("\tYear       : " + year);
            System.err.println("\tComment    : " + comment);
            System.err.println("\tGenre      : " + genre);
        }
    }

    // Part to get MP3 info
    private void getMp3Info() {
        try {
            File fileObj = new File(fullFileName);
            RandomAccessFile rStream = new RandomAccessFile(fileObj, "r");

            long fileLength = fileObj.length();

            // Read integer value (this can be done, because Java has no brobem
            // with small or big endian
            int intMp3Header = rStream.readInt();

            rStream.close();

            //--------------------------------------------------------------------
            // This is a nice demo how to convert byte[] into integer and to swap
            // the byteorder. In addition a HEX value is printed
            /*--------------------------------------------------------------------
             byte mp3Header[] = new byte[4];
             rStream.read(mp3Header);


             rStream.close();

             if(!isSmallEndian)
             {
             byte mp3HeaderTmp;

             mp3HeaderTmp = mp3Header[3];
             mp3Header[3] = mp3Header[0];
             mp3Header[0] = mp3HeaderTmp;

             mp3HeaderTmp = mp3Header[2];
             mp3Header[2] = mp3Header[1];
             mp3Header[1] = mp3HeaderTmp;
             }


             int intMp3Header =   ( mp3Header[0] << 24 )
             | ( ( mp3Header[1] << 16 ) & 0x00ff0000 )
             | ( ( mp3Header[2] <<  8 ) & 0x0000ff00 )
             | ( ( (int) mp3Header[3] ) & 0x000000ff );

             System.out.println("MP3Header:" +  Long.toString(
             ((long)intMp3Header) & 0x00FFFFFFFFL, 16 ) );

             --------------------------------------------------------------------*/
            //System.out.println("Filename: " + fileName);
            //System.out.println("MP3Header:" +  Long.toString(
            //  ((long)intMp3Header) & 0x00FFFFFFFFL, 16 ) );



            int syncWord = (intMp3Header & 0xFFF00000) >> 20;

            int intMpegVers = (intMp3Header & 0x00080000) >> 19;

            mpegVersion = mpegVersionString[intMpegVers];

            int intMpegLayer = (intMp3Header & 0x00060000) >> 17;

            // Has to be layer 3 (correct if not set)
            //if(intMpegLayer != 1)
            //intMpegLayer = 1;

            mpegLayer = mpegLayerString[intMpegLayer];

            int intProtectionBit = (intMp3Header & 0x00010000) >> 16;

            protectionBit = Switch[intProtectionBit];

            int intBitrate = (intMp3Header & 0x0000F000) >> 12;

            int intBitrateFinal = 0;

            if (intMpegLayer != 0)// If layer is unknown we can not calculate
            {
                bitrate = Integer.toString(
                            mpegBitrateString[intMpegVers][intMpegLayer -
                            1][intBitrate]);

                intBitrateFinal = mpegBitrateString[intMpegVers][intMpegLayer -
                        1][intBitrate];
            } else {
                bitrate = "-";
            }
            int intFrequency = (intMp3Header & 0x00000C00) >> 10;

            if (intFrequency > mpegFrequencyString.length)
                frequency = "Unknown";
            else
                frequency = Integer.toString(
                            mpegFrequencyString[intMpegVers][intFrequency]);

            int intPaddingBit = (intMp3Header & 0x00000200) >> 9;

            paddingBit = Switch[intPaddingBit];

            int intPrivateBit = (intMp3Header & 0x00000100) >> 8;

            privateBit = Switch[intPrivateBit];

            int intMpegMode = (intMp3Header & 0x000000C0) >> 6;

            mpegMode = mpegModeString[intMpegMode];

            if (intMpegMode == 1) {
                int intMpegModeExtension = (intMp3Header & 0x00000030) >> 4;

                // If layer 3
                if (intMpegLayer == 1)
                    mpegModeExtension = Integer.toString(
                                mpegModeExtensionString[0][intMpegModeExtension]);
                else
                    mpegModeExtension = Integer.toString(
                                mpegModeExtensionString[1][intMpegModeExtension]);
            }

            int intCopyrightBit = (intMp3Header & 0x00000008) >> 3;

            copyrightBit = Switch[intCopyrightBit];

            int intOriginalHome = (intMp3Header & 0x00000004) >> 2;

            originalHome = Switch[intOriginalHome];

            int intEmphasis = (intMp3Header & 0x00000003);

            emphasis = mpegEmphasisString[intEmphasis];

            if (!bitrate.equals("-") && intMpegLayer == 1) {
                float floatPlayLength = (float) (fileLength /
                        (intBitrateFinal / 8.0 * 1024.0)); // Everything is in Sec

                floatPlayLength *= (float) 1.026; // Somehow we have to calculate this factor to get the
                // right time, no idea why :-(
                int intPlayLengthMin = (int) (floatPlayLength / 60);
                int intPlayLengthSec = ((int) floatPlayLength % 60);

                playLength = Integer.toString((int) floatPlayLength); //.toString(floatPlayLength);
                playLengthMin = Integer.toString(intPlayLengthMin);
                playLengthSec = Integer.toString(intPlayLengthSec);

                if (playLengthMin.length() == 1)
                    playLengthMin = "0" + playLengthMin;

                if (playLengthSec.length() == 1)
                    playLengthSec = "0" + playLengthSec;
            } else {
                playLength = "-";
                playLengthMin = "-";
                playLengthSec = "-";
            }

        } catch (Exception e) {
            System.err.println("Exception in getMp3Info: " + fileName);
            System.out.println("DIR: " + directoryName);
        }

    }// End of getMp3Info

    private void makeMp3Hash() {
        int count = 0;

        mp3Vector.add(mpegVersion);
        mp3Hash.put("Mpeg version", new Integer(count++));

        mp3Vector.add(mpegLayer);
        mp3Hash.put("Mpeg layer", new Integer(count++));

        mp3Vector.add(protectionBit);
        mp3Hash.put("Protection bit", new Integer(count++));

        mp3Vector.add(bitrate);
        mp3Hash.put("Bitrate", new Integer(count++));

        mp3Vector.add(frequency);
        mp3Hash.put("Frequency", new Integer(count++));

        mp3Vector.add(paddingBit);
        mp3Hash.put("Padding bit", new Integer(count++));

        mp3Vector.add(privateBit);
        mp3Hash.put("Private bit", new Integer(count++));

        mp3Vector.add(mpegMode);
        mp3Hash.put("Mode", new Integer(count++));

        mp3Vector.add(mpegModeExtension);
        mp3Hash.put("Mode extension", new Integer(count++));

        mp3Vector.add(copyrightBit);
        mp3Hash.put("Copyright bit", new Integer(count++));

        mp3Vector.add(originalHome);
        mp3Hash.put("Original home", new Integer(count++));

        mp3Vector.add(emphasis);
        mp3Hash.put("Emphasis", new Integer(count++));

        mp3Vector.add(playLength);
        mp3Hash.put("Playlength (sec)", new Integer(count++));

        String pl = new String(playLengthMin + ":" + playLengthSec);

        mp3Vector.add(pl);
        mp3Hash.put("Playlength (min:sec)", new Integer(count++));

    }

    public String getMp3HeaderValue(String searchString) {
        String result = null;

        if (mp3Hash.containsKey(searchString)) {
            Integer index = (Integer) mp3Hash.get(searchString);

            result = (String) mp3Vector.elementAt(index.intValue());
        }

        return result;

    }
}// End of mp3Indo

